@@ -259,6 +259,7 @@ struct lldpd_port {
 	char			*p_id;
 	int			 p_id_len;
 	char			*p_descr;
+	int			 p_descr_force; /* Description has been forced by user */
 	u_int16_t		 p_mfs;
 
 #ifdef ENABLE_DOT3
@@ -398,6 +399,7 @@ struct lldpd_config {
 	int c_bond_slave_src_mac_type; /* Src mac type in lldp frames over bond
 					  slaves */
 	int c_lldp_portid_type; /* The PortID type */
+	int c_lldp_agent_type;	/* The agent type */
 };
 MARSHAL_BEGIN(lldpd_config)
 MARSHAL_STR(lldpd_config, c_mgmt_pattern)
